/*
 * This file is part of Sponge, licensed under the MIT License (MIT).
 *
 * Copyright (c) SpongePowered <https://www.spongepowered.org>
 * Copyright (c) contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.spongepowered.mod.mixin.core.item;

import net.minecraft.block.Block;
import net.minecraft.block.BlockLiquid;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemColored;
import net.minecraft.item.ItemLilyPad;
import net.minecraft.item.ItemStack;
import net.minecraft.stats.StatList;
import net.minecraft.util.BlockPos;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.world.World;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Overwrite;

@Mixin(ItemLilyPad.class)
public abstract class MixinItemLilyPad extends ItemColored {

    public MixinItemLilyPad(Block block, boolean hasSubtypes) {
        super(block, hasSubtypes);
    }

    /**
     * @author bloodmc - December 16th, 2015
     *
     *         Purpose: Restored to vanilla method as we do not need Forge's
     *         block snapshot calls nor event call due to our tracking system.
     */
    @Override
    @Overwrite
    public ItemStack func_77659_a(ItemStack itemStackIn, World worldIn, EntityPlayer playerIn) {
        MovingObjectPosition movingobjectposition = this.func_77621_a(worldIn, playerIn, true);

        if (movingobjectposition == null) {
            return itemStackIn;
        } else {
            if (movingobjectposition.field_72313_a == MovingObjectPosition.MovingObjectType.BLOCK) {
                BlockPos blockpos = movingobjectposition.func_178782_a();

                if (!worldIn.func_175660_a(playerIn, blockpos)) {
                    return itemStackIn;
                }

                if (!playerIn.func_175151_a(blockpos.func_177972_a(movingobjectposition.field_178784_b), movingobjectposition.field_178784_b, itemStackIn)) {
                    return itemStackIn;
                }

                BlockPos blockpos1 = blockpos.func_177984_a();
                IBlockState iblockstate = worldIn.func_180495_p(blockpos);

                if (iblockstate.func_177230_c().func_149688_o() == Material.field_151586_h && ((Integer) iblockstate.func_177229_b(BlockLiquid.field_176367_b)).intValue() == 0
                        && worldIn.func_175623_d(blockpos1)) {
                    worldIn.func_175656_a(blockpos1, Blocks.field_150392_bi.func_176223_P());

                    if (!playerIn.field_71075_bZ.field_75098_d) {
                        --itemStackIn.field_77994_a;
                    }

                    playerIn.func_71029_a(StatList.field_75929_E[Item.func_150891_b(this)]);
                }
            }

            return itemStackIn;
        }
    }
}
